import os
import re
import logging

from lxml import etree
from pelican import signals, contents
from PIL import Image, ImageFilter

from .utils import modify_html, wrap_element


logger = logging.getLogger(__name__)


IMG_MAX_SIZE = 1024
IMG_CLICK_TO_ENLARGE_THRESHOLD = 700

THUMBNAILS_PATH = 'thumbnails'
THUMBNAIL_SAVE_OPTIONS = {
    'quality': 100,
    'optimize': True,
    'progressive': True,
}


def register():
    signals.content_object_init.connect(process_media)


def process_media(content):
    if not isinstance(content, contents.Article):
        return

    with modify_html(content) as html_tree:
        content_dir = content.settings['PATH']

        for iframe in html_tree.xpath('//iframe'):
            iframe_to_figure(iframe)

        for object in html_tree.xpath('//object'):
            object_to_figure(object)

        for blockquote in html_tree.xpath('//blockquote'):
            tweet_to_figure(blockquote)

        for img in html_tree.xpath('//p[count(img) = 1]/img'):
            img_to_figure(img, content_dir)

        for img in html_tree.xpath('//img'):
            create_img_thumbnail(img, content_dir)
            update_img_classes(img)


def iframe_to_figure(iframe):
    wrap_element(iframe, etree.Element('figure'))


def object_to_figure(object):
    wrap_element(object, etree.Element('figure'))


def img_to_figure(img, content_dir):
    if 'left' in img.classes or 'right' in img.classes:
        return

    figure = img.getparent()
    figure.tag = 'figure'
    figure.classes.add('figure')
    img.classes.update(['figure-img', 'img-fluid'])

    create_figcaption(img, figure)


def tweet_to_figure(blockquote):
    if 'twitter-tweet' not in blockquote.classes:
        return

    wrap_element(blockquote, etree.Element('figure'))


def create_figcaption(img, figure):
    figcaption = etree.Element('figcaption')
    create = False

    if img.tail:
        create = True
        figcaption.text = img.tail
        img.tail = ''

    for sibling in img.itersiblings():
        create = True
        figcaption.append(sibling)

    if not create:
        return

    figcaption.set('class', 'figure-caption')
    figure.append(figcaption)


def update_img_classes(img):
    if 'right' in img.classes:
        img.classes.add('pull-xs-right')
    if 'left' in img.classes:
        img.classes.add('pull-xs-left')
    img.classes.update(['img-fluid', 'img-rounded'])


def create_img_thumbnail(img, content_dir):
    img_src = img.get('src')
    if img_src.startswith('http'):
        logger.warning('Found remotely linked image: %s', img_src)
    else:
        filename = get_image_filename(content_dir, img_src)
        info = get_image_info(filename)
        if info['is_image']:
            if info['needs_click_to_enlarge']:
                wrap_element(img, etree.Element('a', attrib={
                    'href': img_src,
                    'target': '_blank',
                    'title': img.get('alt'),
                }))
            if info['needs_thumbnail']:
                tn_filename = create_thumbnail(filename)
                img.set('src', get_image_src(tn_filename))
        else:
            logger.error('Found non-existing image: %s', img_src)


def get_image_info(filename):
    try:
        width, height = Image.open(filename).size
    except IOError:
        return {'is_image': False}
    return {
        'is_image': True,
        'width': width,
        'height': height,
        'needs_thumbnail': (
            width > IMG_MAX_SIZE or
            height > IMG_MAX_SIZE
        ),
        'needs_click_to_enlarge': (
            width > IMG_CLICK_TO_ENLARGE_THRESHOLD or
            height > IMG_CLICK_TO_ENLARGE_THRESHOLD
        ),
    }


def get_image_filename(content_dir, img_src):
    return os.path.join(content_dir, re.sub(r'.*/images/', 'images/', img_src))


def get_image_src(filename):
    return re.sub(r'.*/images/', '{filename}/images/', filename)


def create_thumbnail(filename):
    tn_dir = os.path.join(os.path.dirname(filename), THUMBNAILS_PATH)
    tn_filename = os.path.join(tn_dir, os.path.basename(filename))

    if not os.path.isfile(tn_filename):
        logger.info('Creating thumbnail for %s', filename)
        os.makedirs(tn_dir, exist_ok=True)
        image = Image.open(filename)
        image.thumbnail((IMG_MAX_SIZE, IMG_MAX_SIZE))
        image.filter(ImageFilter.SHARPEN)
        image.save(tn_filename, image.format, **THUMBNAIL_SAVE_OPTIONS)
    return tn_filename



import os


AUTHOR = 'Honza Javorek'

# ABOUT = 'This blog is about kittens and other cute animals.'
# ABOUT_IMAGE = '/images/kitty.jpg'
# ABOUT_HEADING = 'Honza'
# DEFAULT_IMAGE = '/images/default-article-image.jpg'

# SITENAME = 'My Blog'
# SITESUBTITLE = 'The best blog.'

# TWITTER_USERNAME_SITE = 'napyvo'
# TWITTER_USERNAME_AUTHOR = 'honzajavorek'


# Timezone, language
TIMEZONE = 'Europe/Prague'
LOCALE = 'cs_CZ.UTF-8'
DEFAULT_LANG = 'cs'
DEFAULT_DATE_FORMAT = '%x'


# Blog settings
PATH = 'content'
DEFAULT_PAGINATION = False
DEFAULT_CATEGORY = 'blog'
MARKDOWN = {
    'extension_configs': {
        'markdown.extensions.codehilite': {'css_class': 'highlight'},
        'markdown.extensions.headerid': {},
        'markdown.extensions.extra': {},
    },
    'output_format': 'html5',
}


# Outdated articles
OUTDATED_ARTICLE_WARNING = None
OUTDATED_ARTICLE_YEARS = 2


# URL and save paths settings
ARTICLE_URL = '{slug}'
ARTICLE_SAVE_AS = '{slug}.html'
ARTICLE_LANG_URL = '{slug}-{lang}'
ARTICLE_LANG_SAVE_AS = '{slug}-{lang}.html'
PAGE_URL = '{slug}'
PAGE_SAVE_AS = '{slug}.html'
PAGE_LANG_URL = '{slug}-{lang}'
PAGE_LANG_SAVE_AS = '{slug}-{lang}.html'
URL_EXT = ''
FILENAME_METADATA = r'(?P<date>\d{4}-\d{2}-\d{2})_(?P<slug>.*)'


# Static paths will be copied under the same name
STATIC_PATHS = [
    'images',
    'files',
    'robots.txt',
    'favicon.ico',
    'CNAME',
    'site.css',
]
EXTRA_PATH_METADATA = {
    'site.css': {'path': 'theme/css/site.css'},
}
USE_SITE_CSS = True


# Generating
IGNORE_FILES = ['.#*', '.DS_Store']
DELETE_OUTPUT_DIRECTORY = True


# Feeds
FEED_ALL_ATOM = 'feed.xml'
FEED_MAX_ITEMS = 50

CATEGORY_FEED_ATOM = None
TRANSLATION_FEED_ATOM = None
AUTHOR_FEED_ATOM = None
AUTHOR_FEED_RSS = None


# Plugins
from .plugins import PLUGINS  # NOQA


# Theming
THEME = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'theme')

from .jinja_filters import JINJA_FILTERS  # NOQA
THEME_STATIC_PATHS = ['static']

GOOGLE_FONTS = ['Roboto Slab', 'Slabo 13px']


# Development
if os.environ.get('PRODUCTION'):
    PRODUCTION = True
    DEVELOPMENT = False

    RELATIVE_URLS = False
else:
    PRODUCTION = False
    DEVELOPMENT = True

    SITEURL = 'http://localhost:8000'
    RELATIVE_URLS = True

    URL_EXT = '.html'
    ARTICLE_URL += URL_EXT
    ARTICLE_LANG_URL += URL_EXT
    PAGE_URL += URL_EXT
    PAGE_LANG_URL += URL_EXT



import os
import shlex
import random
import subprocess

import click


COMMIT_EMOJIS = [
    ':closed_book:',
    ':green_book:',
    ':blue_book:',
    ':orange_book:',
    ':notebook:',
    ':notebook_with_decorative_cover:',
    ':ledger:',
    ':books:',
    ':pencil2:',
    ':black_nib:',
    ':book:',
    ':memo:',
    ':pencil:',
]


def choose_commit_emoji():
    return random.choice(COMMIT_EMOJIS)


def run(command, env=None, bg=False, capture=False):
    options = {
        'env': dict(os.environ, **env) if env else os.environ,
        'universal_newlines': True,
    }
    if capture:
        options['stdout'] = subprocess.PIPE
        options['stderr'] = subprocess.STDOUT
    if bg:
        return subprocess.Popen(shlex.split(command), **options)

    options['check'] = True
    proc = subprocess.run(shlex.split(command), **options)
    return proc.stdout


def header(text):
    click.secho(text, fg='yellow', bold=True)


def abort(context):
    click.echo('Aborted!')
    context.exit(1)


def pelican(config, *extra_params, production=False):
    os.makedirs(config['OUTPUT_DIR'], exist_ok=True)

    command = 'pelican "{content}" --output="{output}" --settings="{settings}"'
    if extra_params:
        command = ' '.join([command] + list(extra_params))

    command = command.format(
        content=config['CONTENT_DIR'],
        output=config['OUTPUT_DIR'],
        settings=os.path.join(config['CWD'], config['SETTINGS_PATH']),
    )
    run(command, env={'PRODUCTION': '1'} if production else None)



import re
import json
import urllib.parse
from datetime import date, datetime

import markdown as md
from jinja2 import Markup


JINJA_FILTERS = {}


def register_filter(func):
    JINJA_FILTERS[func.__name__] = func
    return func


@register_filter
def urlencode(s):
    return urllib.parse.quote(s)


@register_filter
def markdown(s):
    return Markup(md.markdown(s, output_format='html5'))


@register_filter
def to_datetime(dt):
    if not isinstance(dt, datetime):
        dt = str(dt)
        try:
            return datetime.strptime(dt, '%Y-%m-%d %H:%M:%S')
        except ValueError:
            return datetime.strptime(dt, '%Y-%m-%d')
    return dt


@register_filter
def month_name(month_no):
    return [
        'leden', 'únor', 'březen',
        'duben', 'květen', 'červen',
        'červenec', 'srpen', 'září',
        'říjen', 'listopad', 'prosinec',
    ][month_no - 1]


@register_filter
def format_date(dt, format, strip_zeros=True):
    formatted = to_datetime(dt).strftime(format)
    if strip_zeros:
        return re.sub(r'\b0', '', formatted)
    return formatted


@register_filter
def copyright(articles):
    current_year = date.today().year
    years = sorted(article.date.year for article in articles)
    try:
        since_year = years[0]
    except KeyError:
        since_year = current_year

    if current_year == since_year:
        return '© {}'.format(current_year)
    return '© {}—{}'.format(since_year, current_year)


@register_filter
def tojson(*args, **kwargs):
    return json.dumps(*args, **kwargs).replace('/', '\\/')


@register_filter
def has_images(html):
    return re.search(r'<img[^\>]*>', html)


@register_filter
def to_css_class(s):
    return s.replace('_', '-').replace('/', '-')


@register_filter
def prevent_line_breaks(s):
    parts = []
    words = s.split()
    for i, (word1, word2) in enumerate(zip(words, words[1:] + [None])):
        parts.append(word1)
        if word2:
            if len(word2) <= 3:
                parts.append('&nbsp;')
            else:
                parts.append(' ')

    return Markup(''.join(parts))


@register_filter
def google_fonts(fonts):
    url = 'https://fonts.googleapis.com/css?family='
    url += '|'.join([
        urllib.parse.quote_plus(font) + ':400,700' for font in fonts
    ])
    url += '&amp;subset=latin,latin-ext'
    return url



import re
import subprocess

import click
import requests

from . import blog
from .helpers import choose_commit_emoji, run, header, abort


@blog.command()
@click.pass_context
def publish(context):
    """Saves changes and sends them to GitHub"""

    header('Recording changes...')
    run('git add -A')

    header('Displaying changes...')
    run('git -c color.status=always status')

    if not click.confirm('\nContinue publishing'):
        run('git reset HEAD --')
        abort(context)

    header('Saving changes...')
    try:
        run('git commit -m "{message}"'.format(
            message='Publishing {}'.format(choose_commit_emoji())
        ), capture=True)
    except subprocess.CalledProcessError as e:
        if 'nothing to commit' not in e.stdout:
            raise
        else:
            click.echo('Nothing to commit.')

    header('Pushing to GitHub...')
    branch = get_branch()
    run('git push origin {branch}:{branch}'.format(branch=branch))

    pr_link = get_pr_link(branch)
    if pr_link:
        click.launch(pr_link)


def get_pr_link(branch):
    repo_slug = get_repo_slug()
    repo_url = 'https://api.github.com/repos/{}'.format(repo_slug)

    res = requests.get(repo_url)
    res.raise_for_status()
    repo_info = res.json()

    if not repo_info['fork']:
        return None

    return 'https://github.com/{}/compare/master...{}:{}'.format(
        repo_info['parent']['full_name'],
        repo_info['owner']['login'],
        branch,
    )


def get_repo_slug():
    url = run('git remote get-url origin', capture=True)
    return re.search(r'github\.com[\:\/](.+)\.git$', url).group(1)


def get_branch():
    branches = run('git branch --no-color', capture=True)
    return re.search(r'\*\s+(\S+)', branches).group(1)



import os
import time

import click

from . import blog
from .helpers import run, abort, pelican


PORT = 8000


@blog.command()
@click.pass_context
def preview(context):
    """Opens local preview of your blog website"""

    config = context.obj

    pelican(config, '--verbose', '--ignore-cache')

    server_proc = None
    os.chdir(config['OUTPUT_DIR'])
    try:
        try:
            command = 'python -m http.server ' + str(PORT)
            server_proc = run(command, bg=True)

            time.sleep(3)
            click.launch('http://localhost:8000')

            time.sleep(5)
            pelican(config, '--autoreload')
        except Exception:
            if server_proc is not None:
                server_proc.kill()
            raise
    except KeyboardInterrupt:
        abort(context)



import pytz
import datetime

from pelican import signals, contents


def register():
    signals.content_object_init.connect(calc_outdated_article)


def calc_outdated_article(content):
    if not isinstance(content, contents.Article):
        return

    warning = content.settings.get('OUTDATED_ARTICLE_WARNING')
    years = content.settings.get('OUTDATED_ARTICLE_YEARS')

    if warning and years:
        now = datetime.datetime.utcnow().replace(tzinfo=pytz.utc)
        years_td = datetime.timedelta(days=365 * years)

        content.is_outdated = content.date < now - years_td
    else:
        content.is_outdated = False



import os
import importlib.util

import click
from pelican.settings import DEFAULT_CONFIG


@click.group()
@click.version_option()
@click.pass_context
def blog(context):
    cwd = os.getcwd()

    settings_path = 'settings.py'
    settings_file = os.path.join(cwd, settings_path)

    config = {'CWD': cwd, 'SETTINGS_PATH': settings_path}
    config.update(DEFAULT_CONFIG)
    config.update(load_settings_file_as_dict(settings_file))

    config['CONTENT_DIR'] = os.path.join(cwd, config['PATH'])
    config['OUTPUT_DIR'] = os.path.join(cwd, config['OUTPUT_PATH'])

    context.obj = config


from . import (  # NOQA
    update,
    write,
    photos,
    preview,
    publish,
    lint,
    deploy,
)


def load_settings_file_as_dict(filename):
    file_spec = importlib.util.spec_from_file_location('settings', filename)
    module = importlib.util.module_from_spec(file_spec)
    file_spec.loader.exec_module(module)

    config = {}
    for (key, value) in module.__dict__.items():
        if key.upper() == key:
            config[key] = value
    return config



import os
import shutil

import click

from . import blog
from .helpers import choose_commit_emoji, run, header, pelican


@blog.command()
@click.pass_context
def deploy(context):
    """Uploads new version of the blog website"""

    config = context.obj

    header('Generating HTML...')
    pelican(config, '--verbose', production=True)

    header('Removing unnecessary output...')
    unnecessary_paths = [
        'author', 'category', 'drafts', 'tag', 'feeds', 'tags.html',
        'authors.html', 'categories.html', 'archives.html',
    ]
    for path in unnecessary_paths:
        remove_path(os.path.join(config['OUTPUT_DIR'], path))

    if os.environ.get('TRAVIS'):  # Travis CI
        header('Setting up Git...')
        run(
            'git config user.name ' +
            run('git show --format="%cN" -s', capture=True)
        )
        run(
            'git config user.email ' +
            run('git show --format="%cE" -s', capture=True)
        )

        github_token = os.environ.get('GITHUB_TOKEN')
        repo_slug = os.environ.get('TRAVIS_REPO_SLUG')
        origin = 'https://{}@github.com/{}.git'.format(github_token, repo_slug)
        run('git remote set-url origin ' + origin)

    header('Rewriting gh-pages branch...')
    run('ghp-import -m "{message}" {dir}'.format(
        message='Deploying {}'.format(choose_commit_emoji()),
        dir=config['OUTPUT_DIR'],
    ))

    header('Pushing to GitHub...')
    run('git push origin gh-pages:gh-pages --force')


def remove_path(path):
    if os.path.isfile(path):
        os.remove(path)
    else:
        shutil.rmtree(path, ignore_errors=True)



from lxml import etree
from pelican import signals, contents

from .utils import modify_html, wrap_element


def register():
    signals.content_object_init.connect(enhance_tables)


def enhance_tables(content):
    if not isinstance(content, contents.Article):
        return

    with modify_html(content) as html_tree:
        for table in html_tree.findall('.//table'):
            table.classes.update(['table', 'table-bordered'])
            div = etree.Element('div')
            div.set('class', 'table-responsive')
            wrap_element(table, div)



from . import (
    code_blocks, headings, plain_summary, featured_image, media, tables,
    authors, outdated_article,
)


PLUGINS = [
    code_blocks, headings, plain_summary, featured_image, media, tables,
    authors, outdated_article
]



import re
import contextlib

from lxml import html


@contextlib.contextmanager
def modify_html(content, prop='_content'):
    html_string = getattr(content, prop)
    html_tree = html.fromstring(html_string)

    yield html_tree

    html_string = html.tostring(html_tree, encoding='unicode')
    html_string = re.sub(r'%7B(\w+)%7D', r'{\1}', html_string)
    html_string = re.sub(r'%7C(\w+)%7C', r'|\1|', html_string)
    setattr(content, prop, html_string)


def wrap_element(element, wrapper_element):
    element.addprevious(wrapper_element)
    wrapper_element.insert(0, element)
    return wrapper_element



import os
import hashlib
import urllib.parse

import requests
from pelican import signals, contents


GRAVATAR_SIZE = 200


def register():
    signals.content_object_init.connect(process_author_info)


def process_author_info(content):
    if not isinstance(content, contents.Article):
        return

    process_twitter(content)
    process_gravatar(content)
    process_about(content)


def process_gravatar(content):
    gravatar = getattr(content, 'gravatar', None)
    if gravatar:
        params = {}

        if content.twitter:
            url = 'https://twitter.com/{}/profile_image?size=original'
            url = url.format(content.twitter)
            try:
                resp = requests.head(url)
                resp.raise_for_status()
                params['d'] = resp.headers['location']
            except Exception:
                pass

        if not params.get('d') and 'DEFAULT_GRAVATAR' in content.settings:
            default_gravatar_url = os.path.join(
                content.settings['SITEURL'],
                content.settings['DEFAULT_GRAVATAR']
            )
            params['d'] = default_gravatar_url

        params['s'] = str(content.settings.get('GRAVATAR_SIZE', GRAVATAR_SIZE))
        gravatar_url = (
            'http://www.gravatar.com/avatar/' +
            hashlib.md5(gravatar.lower().encode('utf-8')).hexdigest()
        )
        gravatar_url += '?' + urllib.parse.urlencode(params)
        content.gravatar = gravatar_url
    else:
        content.gravatar = None


def process_about(content):
    content.about = (
        getattr(content, 'about', None) or
        content.settings.get('ABOUT') or
        None
    )

    about_image = (
        getattr(content, 'aboutimage', None) or
        content.settings.get('ABOUT_IMAGE') or
        None
    )
    content.aboutimage = about_image
    content.about_image = about_image


def process_twitter(content):
    content.twitter = (
        getattr(content, 'twitter', None) or
        content.settings.get('TWITTER_USERNAME_AUTHOR') or
        None
    )



from . import blog
from .helpers import run


@blog.command()
def update():
    """Gets other people's changes from GitHub"""

    run('git pull --rebase origin master')
    run('pip install -r requirements.txt')



import re
import os
from glob import glob

import click
from PIL import Image, ImageFilter, ImageSequence

from . import blog
from .helpers import header, abort


IMAGE_MAX_SIZE = 1900
IMAGE_SAVE_OPTIONS = {
    'quality': 100,
    'optimize': True,
    'progressive': True,
}
IMAGES_PATH = 'images'


@blog.command()
@click.argument('path')
@click.pass_context
def photos(context, path):
    """Adds images to the last article"""

    config = context.obj

    header('Looking for the latest article...')
    article_filename = find_last_article(config['CONTENT_DIR'])
    if not article_filename:
        return click.secho('No articles.', fg='red')
    click.echo(os.path.basename(article_filename))

    header('Looking for images...')
    images = list(find_images(path))
    if not images:
        return click.secho('Found no images.', fg='red')

    for filename in images:
        click.secho(filename, fg='green')

    if not click.confirm('\nAdd these images to the latest article'):
        abort(config)

    url_prefix = os.path.join('{filename}', IMAGES_PATH)
    images_dir = os.path.join(config['CONTENT_DIR'], IMAGES_PATH)
    os.makedirs(images_dir, exist_ok=True)

    header('Processing images...')
    urls = []
    for filename in images:
        image_basename = os.path.basename(filename).replace(' ', '-').lower()
        urls.append(os.path.join(url_prefix, image_basename))
        image_filename = os.path.join(images_dir, image_basename)
        print(filename, image_filename)
        import_image(filename, image_filename)

    content = '\n'
    for url in urls:
        url = url.replace('\\', '/')
        content += '\n![image description]({})\n'.format(url)

    header('Adding to article: {}'.format(article_filename))
    with click.open_file(article_filename, 'a') as f:
        f.write(content)
    click.launch(article_filename)


def import_image(src_filename, dest_filename):
    if os.path.isfile(dest_filename):
        click.echo('Already exists, skipping: {}'.format(dest_filename))
    else:
        with Image.open(src_filename) as image:
            image.thumbnail((IMAGE_MAX_SIZE, IMAGE_MAX_SIZE))

            try:
                image.filter(ImageFilter.SHARPEN)
            except ValueError:
                pass  # skip filtering for images which do not support it

            click.echo('Saving: {}'.format(dest_filename))
            options = dict(IMAGE_SAVE_OPTIONS)
            if is_animated(image):
                options.setdefault('save_all', True)
            image.save(dest_filename, image.format, **options)


def is_animated(image):
    return len(list(ImageSequence.Iterator(image))) > 1


def find_last_article(content_dir):
    articles = list(sorted(glob(os.path.join(content_dir, '*.md'))))
    if not articles:
        return None

    filename = articles[-1]
    date = os.path.basename(filename)[0:10]

    candidates = []
    for filename in articles:
        if os.path.basename(filename)[0:10] == date:
            candidates.append(filename)

    if len(candidates) == 1:
        return candidates[0]

    to_sort = []
    for filename in candidates:
        with click.open_file(filename) as f:
            match = re.search(r'Date: (.+)', f.read())
        creation_date = match.group(1)
        to_sort.append((creation_date, filename))

    return list(sorted(to_sort))[-1][1]


def find_images(path):
    for filename in find_files(path):
        try:
            Image.open(filename).close()
        except IOError:
            continue
        else:
            yield filename


def find_files(path):
    if os.path.isdir(path):
        for root_path, dir_paths, file_paths in os.walk(path):
            yield root_path
            for dir_path in dir_paths:
                yield os.path.join(root_path, dir_path)
            for file_path in file_paths:
                yield os.path.join(root_path, file_path)
    else:
        yield path



import os
from datetime import datetime

import click
from slugify import slugify

from . import blog


@blog.command()
@click.pass_context
def write(context):
    """Starts a new article"""

    config = context.obj

    title = click.prompt('Title')
    author = click.prompt('Author', default=config.get('DEFAULT_AUTHOR'))

    slug = slugify(title)
    creation_date = datetime.now()
    basename = '{:%Y-%m-%d}_{}.md'.format(creation_date, slug)
    meta = (
        ('Title', title),
        ('Date', '{:%Y-%m-%d %H:%M}:00'.format(creation_date)),
        ('Modified', '{:%Y-%m-%d %H:%M}:00'.format(creation_date)),
        ('Author', author),
    )

    file_content = ''
    for key, value in meta:
        file_content += '{}: {}\n'.format(key, value)
    file_content += '\n\n'
    file_content += 'Text...\n\n'
    file_content += '![image description]({filename}/images/my-photo.jpg)\n\n'
    file_content += 'Text...\n\n'

    os.makedirs(config['CONTENT_DIR'], exist_ok=True)
    path = os.path.join(config['CONTENT_DIR'], basename)
    with click.open_file(path, 'w') as f:
        f.write(file_content)

    click.echo(path)
    click.launch(path)



from pelican import signals, contents

from .utils import modify_html


def register():
    signals.content_object_init.connect(process_code_blocks)


def process_code_blocks(content):
    if not isinstance(content, contents.Article):
        return

    with modify_html(content) as html_tree:
        for pre in html_tree.findall('.//div/pre'):
            pre.getparent().tag = 'pre'
            pre.tag = 'code'



from pelican import signals, contents

from .utils import modify_html


def register():
    signals.content_object_init.connect(make_summary_plain)


def make_summary_plain(content):
    if not isinstance(content, contents.Article):
        return

    content._summary = content._content
    with modify_html(content, prop='_summary') as html_tree:
        queries = [
            '//iframe', '//object', '//p[count(img) = 1]',
            '//img', '//figure',
        ]
        for query in queries:
            for element in html_tree.xpath(query):
                element.getparent().remove(element)



import os

from pelican import signals, contents

from .utils import modify_html


def register():
    signals.content_object_init.connect(attach_featured_image)


def attach_featured_image(content):
    if not isinstance(content, contents.Article):
        return

    image = getattr(content, 'image', None)
    if not image:
        with modify_html(content) as html_tree:
            try:
                image = html_tree.findall('.//img')[0].get('src')
            except IndexError:
                image = content.settings.get('DEFAULT_IMAGE')
                if image:
                    image = os.path.join(content.settings['SITEURL'], image)

    if image:
        content.image = image.format(filename=content.get_siteurl())
    else:
        content.image = None



import sys
from setuptools import setup, find_packages

try:
    from semantic_release import setup_hook
    setup_hook(sys.argv)
except ImportError:
    message = "Unable to locate 'semantic_release', releasing won't work"
    print(message, file=sys.stderr)


version = '1.3.2'


install_requires = [
    'pelican',
    'ghp-import',
    'lxml',
    'pillow',
    'python-slugify',
    'click',
    'markdown',
    'colorama',
    'requests',
    'flake8',  # intentionally also here - used by 'blog lint'
]
tests_require = [
    'pytest-runner',
    'pytest',
    'pytest-cov',
    'coveralls',
    'flake8',
]
release_requires = [
    'python-semantic-release',
]


setup(
    name='danube-delta',
    version=version,
    description='Honza Javorek\'s Pelican setup',
    long_description=open('README.rst').read(),
    author='Honza Javorek',
    author_email='mail@honzajavorek.cz',
    url='http://github.com/honzajavorek/danube-delta',
    license=open('LICENSE').read(),
    packages=find_packages(),
    include_package_data=True,
    install_requires=install_requires,
    tests_require=tests_require,
    extras_require={
        'tests': tests_require,
        'release': release_requires,
    },
    entry_points={
        'console_scripts': [
            'blog = danube_delta.cli:blog'
        ]
    },
    classifiers=(
        'Environment :: Console',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
    ),
    keywords='pelican blog',
)



from lxml import etree
from pelican import signals, contents

from .utils import modify_html


def register():
    signals.content_object_init.connect(enhance_headings)


def enhance_headings(content):
    if not isinstance(content, contents.Article):
        return

    with modify_html(content) as html_tree:
        for query in ['.//h1[@id]', './/h2[@id]', './/h3[@id]', './/h4[@id]']:
            for heading in html_tree.findall(query):
                add_permalink(heading)


def add_permalink(heading):
    a = etree.Element('a')
    a.text = '#'
    a.set('href', '#{}'.format(heading.get('id')))
    a.set('title', 'Trvalý odkaz na tento nadpis')
    small = etree.Element('small')
    small.set('class', 'permalink')
    small.append(a)
    heading.append(small)



from subprocess import SubprocessError

import click

from . import blog
from .helpers import run


EXCLUDE = ['.git', '__pycache', 'env', 'venv', 'settings.py']


@blog.command()
@click.pass_context
def lint(context):
    """Looks for errors in source code of your blog"""

    config = context.obj
    try:
        run('flake8 {dir} --exclude={exclude}'.format(
            dir=config['CWD'],
            exclude=','.join(EXCLUDE),
        ))
    except SubprocessError:
        context.exit(1)
